<?php
/**
 * NOTICE OF LICENSE
 *
 * All right is reserved,
 * Please go through LICENSE.txt file inside our module
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future. If you wish to customize this module for your
 * needs please refer to CustomizationPolicy.txt file inside our module for more information.
 *
 * @author Webkul IN
 * @copyright since 2010 Webkul
 * @license LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Adapter\SymfonyContainer;
use PrestaShop\PrestaShop\Core\FeatureFlag\FeatureFlagSettings;

class WkProductDownload extends ObjectModel
{
    public $id_product;
    public $id_product_attribute;
    public $display_filename;
    public $filename;
    public $date_expiration;
    public $nb_days_accessible;
    public $nb_downloadable;
    public $is_shareable;
    public $is_virtual_file;

    public static $definition = [
        'table' => 'wk_product_download',
        'primary' => 'id_wk_product_download',
        'multilang' => false,
        'multilang_shop' => true,
        'fields' => [
            'id_product' => ['type' => self::TYPE_INT, 'required' => true, 'shop' => true],
            'id_product_attribute' => ['type' => self::TYPE_INT, 'required' => true, 'shop' => true],
            'nb_days_accessible' => ['type' => self::TYPE_INT, 'required' => false, 'shop' => true],
            'nb_downloadable' => ['type' => self::TYPE_INT, 'required' => false, 'shop' => true],
            'display_filename' => ['type' => self::TYPE_STRING, 'size' => 255, 'shop' => true],
            'filename' => ['type' => self::TYPE_STRING, 'size' => 255, 'shop' => true],
            'active' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'shop' => true],
            'is_shareable' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'shop' => true],
            'is_virtual_file' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool', 'shop' => true],
            'date_expiration' => [
                'type' => self::TYPE_DATE,
                'validate' => 'isDateFormat',
                'required' => false,
                'shop' => true,
            ],
        ],
    ];

    public function __construct($id = null, $idLang = null, $idShop = null)
    {
        Shop::addTableAssociation(
            'wk_product_download',
            ['type' => 'shop', 'primary' => 'id_wk_product_download']
        );
        parent::__construct($id, $idLang, $idShop);
    }

    public static function getWkProductDownload($productId, $attributeId)
    {
        if (!empty($productId) && !empty($attributeId)) {
            $shopId = Context::getContext()->shop->id;

            return Db::getInstance()->getRow(
                'SELECT pds.* FROM `' . _DB_PREFIX_ . 'wk_product_download` pd
                LEFT JOIN `' . _DB_PREFIX_ . 'wk_product_download_shop` pds
                ON (pd.`id_wk_product_download` = pds.`id_wk_product_download`)
                WHERE pds.`id_product` =   ' . (int) $productId . '
                AND pds.`id_product_attribute` =  ' . (int) $attributeId . '
                AND pds.`id_shop` =  ' . (int) $shopId . ''
            );
        }
    }

    public function getTextLink($idOrderDetail)
    {
        $result = Db::getInstance()->getRow(
            'SELECT * FROM `' . _DB_PREFIX_ . 'wk_virtual_order`
            WHERE `id_order_detail` =  ' . (int) $idOrderDetail . '  AND
            `id_product` =  ' . (int) $this->id_product . '  AND
            `id_product_attribute` =  ' . (int) $this->id_product_attribute . ''
        );
        $downloadLink = null;
        if ($result) {
            $key = $this->filename . '-' . $result['download_hash'];
            $downloadLink = Context::getContext()->link->getModuleLink('wkvirtualcombination', 'getfile');
            $downloadLink .= '?&key=' . $key;
        }

        return $downloadLink;
    }

    public static function updateWkProductDownload($productId, $attributeId)
    {
        $shopId = Context::getContext()->shop->id;
        Db::getInstance()->execute(
            'UPDATE `' . _DB_PREFIX_ . 'wk_product_download_shop`
            SET `filename` = "", `is_virtual_file` = 0, `active` = 0
            WHERE `id_product` =   ' . (int) $productId . ' AND `id_product_attribute` =  ' . (int) $attributeId . '
            AND `id_shop` =  ' . (int) $shopId . ''
        );

        return Db::getInstance()->execute(
            'UPDATE `' . _DB_PREFIX_ . 'wk_product_download`
            SET `filename` = "", `is_virtual_file` = 0, `active` = 0
            WHERE `id_product` =   ' . (int) $productId . ' AND `id_product_attribute` =  ' . (int) $attributeId . ''
        );
    }

    public static function getFileTypes()
    {
        return [
            'jpeg',
            'jpg',
            'png',
            'gif',
            'tif',
            'pdf',
            'csv',
            'doc',
            'docx',
            'ppt',
            'pptx',
            'txt',
            '.svg',
            '.ico',
        ];
    }

    /**
     * Return the display filename from a physical filename
     *
     * @param string $filename Filename physically
     *
     * @return string Filename the display filename for this virtual product
     */
    public static function getFilenameFromFilename($filename)
    {
        $shopId = Context::getContext()->shop->id;

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('
		SELECT pds.`display_filename`
		FROM `' . _DB_PREFIX_ . 'wk_product_download` pd
        LEFT JOIN `' . _DB_PREFIX_ . 'wk_product_download_shop` pds
        ON (pd.`id_wk_product_download` = pds.`id_wk_product_download`)
		WHERE pds.`filename` = \'' . pSQL($filename) . '\'  AND pds.`id_shop` =  ' . (int) $shopId . '');
    }

    public static function isEnableNewProductPage()
    {
        if (version_compare(_PS_VERSION_, '1.7.8.9', '>')) {
            if (SymfonyContainer::getInstance()->get('prestashop.core.admin.feature_flag.repository')->isEnabled(FeatureFlagSettings::FEATURE_FLAG_PRODUCT_PAGE_V2) || SymfonyContainer::getInstance()->get('prestashop.core.admin.feature_flag.repository')->isEnabled(FeatureFlagSettings::FEATURE_FLAG_PRODUCT_PAGE_V2_MULTI_SHOP)) {
                return true;
            }
        }
        return false;
    }
}
