<?php
/**
 * NOTICE OF LICENSE
 *
 * All right is reserved,
 * Please go through LICENSE.txt file inside our module
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future. If you wish to customize this module for your
 * needs please refer to CustomizationPolicy.txt file inside our module for more information.
 *
 * @author Webkul IN
 * @copyright since 2010 Webkul
 * @license LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Adapter\Attribute\AdminAttributeGeneratorControllerWrapper;
use PrestaShop\PrestaShop\Adapter\Attribute\AttributeDataProvider;

class WkVirtualCombinationAvailable extends ObjectModel
{
    public $id_product;
    public $availabe;

    public static $definition = [
        'table' => 'wk_virtual_combination_available',
        'primary' => 'id_wk_virtual_combination_available',
        'multilang_shop' => true,
        'fields' => [
            'id_product' => ['type' => self::TYPE_INT, 'required' => true, 'shop' => true],
            'availabe' => ['type' => self::TYPE_INT, 'required' => true, 'shop' => true],
        ],
    ];

    public function __construct($id = null, $idLang = null, $idShop = null)
    {
        Shop::addTableAssociation(
            'wk_virtual_combination_available',
            ['type' => 'shop', 'primary' => 'id_wk_virtual_combination_available']
        );
        parent::__construct($id, $idLang, $idShop);
    }

    public static function getAvailableId($idProduct, $idShop)
    {
        return Db::getInstance()->getValue(
            'SELECT vas.`id_wk_virtual_combination_available`
            FROM `' . _DB_PREFIX_ . 'wk_virtual_combination_available` va
            LEFT JOIN  `' . _DB_PREFIX_ . 'wk_virtual_combination_available_shop` vas ON (va.`id_wk_virtual_combination_available` = vas.`id_wk_virtual_combination_available`)
            WHERE vas.`id_shop` = ' . (int) $idShop . ' AND vas.`id_product` = ' . (int) $idProduct
        );
    }

    public static function availableForVirtualCombi($idProduct, $idShop)
    {
        return Db::getInstance()->getValue(
            'SELECT vas.`availabe`
            FROM `' . _DB_PREFIX_ . 'wk_virtual_combination_available` va
            LEFT JOIN  `' . _DB_PREFIX_ . 'wk_virtual_combination_available_shop` vas ON (va.`id_wk_virtual_combination_available` = vas.`id_wk_virtual_combination_available`)
            WHERE vas.`id_shop` = ' . (int) $idShop . ' AND vas.`id_product` = ' . (int) $idProduct
        );
    }

    public static function updateProductType($idProduct, $updateProductData)
    {
        Db::getInstance()->update('product', $updateProductData, 'id_product = ' . (int) $idProduct);
    }

    public static function deleteAllCombinations($idProduct)
    {
        $combinations = self::getAllCombinationsByProductId($idProduct);
        if (is_array($combinations)) {
            foreach ($combinations as $combination) {
                $wrapperObj = new AdminAttributeGeneratorControllerWrapper();
                $wrapperObj->ajaxProcessDeleteProductAttribute($combination['id_product_attribute'], (int) $idProduct);
            }
        }
    }

    public static function getAllCombinationsByProductId($idProduct)
    {
        $combinations = (new AttributeDataProvider())->getProductCombinations((int) $idProduct);

        return $combinations;
    }

    public static function getAllCominationsName($idProduct, $idLang)
    {
        $combinations = self::getAllCombinationsByProductId($idProduct);
        $combinationData = [];
        if (is_array($combinations)) {
            $context = Context::getContext();
            foreach ($combinations as $key => $combination) {
                $query = new DbQuery();
                $query->select('CONCAT(GROUP_CONCAT(DISTINCT agl.`name`, \' - \', al.name SEPARATOR \', \')) as name');
                $query->from('product_attribute', 'pa');
                $query->join(Shop::addSqlAssociation('product_attribute', 'pa'));
                $query->innerJoin('product_lang', 'pl', 'pl.id_product = pa.id_product
                AND pl.id_lang = ' . (int) $idLang);
                $query->leftJoin(
                    'product_attribute_combination',
                    'pac',
                    'pac.id_product_attribute = pa.id_product_attribute'
                );
                $query->leftJoin('attribute', 'atr', 'atr.id_attribute = pac.id_attribute');
                $query->leftJoin(
                    'attribute_lang',
                    'al',
                    'al.id_attribute = atr.id_attribute AND al.id_lang = ' . (int) $idLang
                );
                $query->leftJoin('attribute_group_lang', 'agl', 'agl.id_attribute_group = atr.id_attribute_group
                AND agl.id_lang = ' . (int) $idLang);
                $query->where('pa.id_product = ' . (int) $idProduct . '
                AND pa.id_product_attribute = ' . (int) $combination['id_product_attribute']);
                $name = Db::getInstance()->getValue($query);
                if ($name) {
                    $virtualFile = WkProductDownload::getWkProductDownload(
                        $idProduct,
                        $combination['id_product_attribute']
                    );
                    $downloadLink = '';
                    if (!empty($virtualFile) && !empty($virtualFile['filename'])) {
                        $downloadLink = $context->link->getModuleLink('wkvirtualcombination', 'getfile')
                        . '?&file=' .
                        $virtualFile['filename'];
                        $virtualFile['download_link'] = $downloadLink;
                    }
                    if (isset($virtualFile['date_expiration'])) {
                        $virtualFile['date_expiration'] = date('Y-m-d', strtotime($virtualFile['date_expiration']));
                    }
                    // dump($virtualFile);die;
                    $combinationData[$key]['name'] = $name;
                    $combinationData[$key]['id_product_attribute'] = $combination['id_product_attribute'];
                    $combinationData[$key]['virtual_file_data'] = $virtualFile;
                }
            }
        }

        return $combinationData;
    }
}
