<?php
/**
 * NOTICE OF LICENSE
 *
 * All right is reserved,
 * Please go through LICENSE.txt file inside our module
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future. If you wish to customize this module for your
 * needs please refer to CustomizationPolicy.txt file inside our module for more information.
 *
 * @author Webkul IN
 * @copyright since 2010 Webkul
 * @license LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class WkVirtualOrder extends ObjectModel
{
    public $idProduct;
    public $idOrderDetail;
    public $idProductAttribute;
    public $downloadNb;
    public $downloadHash;
    public $downloadDeadline;

    public static $definition = [
        'table' => 'wk_virtual_order',
        'primary' => 'id_wk_virtual_order',
        'multilang' => false,
        'fields' => [
            'id_product' => ['type' => self::TYPE_INT],
            'id_order_detail' => ['type' => self::TYPE_INT],
            'id_product_attribute' => ['type' => self::TYPE_INT],
            'download_nb' => ['type' => self::TYPE_INT],
            'download_hash' => ['type' => self::TYPE_STRING, 'size' => 255],
            'download_deadline' => ['type' => self::TYPE_DATE, 'validate' => 'isDateFormat'],
        ],
    ];

    public static function getVirtualOrder($idOrderDetail)
    {
        $result = Db::getInstance()->getRow('SELECT * FROM `' . _DB_PREFIX_ . 'wk_virtual_order`
        WHERE id_order_detail = ' . (int) $idOrderDetail);

        if ($result) {
            $virtualOrder = new WkVirtualOrder($result['id_wk_virtual_order']);
        } else {
            $virtualOrder = new WkVirtualOrder();
        }

        return $virtualOrder;
    }

    /**
     * Return a deadline.
     *
     * @return string Datetime in SQL format
     */
    public static function getDeadline($productId, $attributeId)
    {
        $productDownload = WkProductDownload::getWkProductDownload($productId, $attributeId);

        if ($productDownload) {
            if (!(int) $productDownload['nb_days_accessible']) {
                return '0000-00-00 00:00:00';
            }
            $timestamp = strtotime('+' . (int) $productDownload['nb_days_accessible'] . ' day');

            return date('Y-m-d H:i:s', $timestamp);
        }
    }

    public static function getDownloadFromHash($hash)
    {
        if ($hash == '') {
            return false;
        }

        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('SELECT *
        FROM `' . _DB_PREFIX_ . 'wk_virtual_order`
        WHERE `download_hash` = \'' . pSQL($hash) . '\'');
    }

    public static function incrementDownload($idProductDownload, $increment = 1)
    {
        $sql = 'UPDATE `' . _DB_PREFIX_ . 'wk_virtual_order`
            SET `download_nb` = `download_nb` + ' . (int) $increment . '
            WHERE `id_wk_virtual_order`= ' . (int) $idProductDownload . '
            LIMIT 1';

        return Db::getInstance()->execute($sql);
    }
}
