<?php
/**
 * NOTICE OF LICENSE
 *
 * All right is reserved,
 * Please go through LICENSE.txt file inside our module
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future. If you wish to customize this module for your
 * needs please refer to CustomizationPolicy.txt file inside our module for more information.
 *
 * @author Webkul IN
 * @copyright since 2010 Webkul
 * @license LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Domain\Product\ValueObject\ProductType;

class AdminWkVirtualCombinationController extends ModuleAdminController
{
    protected $id_current_category;

    public function __construct()
    {
        $this->bootstrap = true;
        $this->table = 'product';
        $this->className = 'Product';
        $this->lang = true;

        if (!Tools::getValue('id_product')) {
            $this->multishop_context_group = false;
        }

        parent::__construct();
        $this->toolbar_title = $this->l('Virtual Combination File');
        $this->page_header_toolbar_title = $this->l('Virtual Combination File');
        $this->imageType = 'jpg';
        $this->_defaultOrderBy = 'position';

        $this->_conf[33] = $this->l('Virtual combination enabled. To create a combination, please ');
        if (Tools::getValue('id_p')) {
            $this->context->smarty->assign([
                'p_link' => $this->boProductPageLink(Tools::getValue('id_p')),
            ]);
            $this->_conf[33] .= $this->context->smarty->fetch(
                _PS_MODULE_DIR_ . $this->module->name .
                '/views/templates/admin/_partials/admin_product_link.tpl'
            );
        }
        $this->_conf[34] = $this->l('Virtual combination disabled.');
        if (Tools::getValue('reset_filter_category')) {
            $this->context->cookie->id_category_products_filter = false;
        }

        $this->_join .= '
        LEFT JOIN `' . _DB_PREFIX_ . 'stock_available` sav ON (sav.`id_product` = a.`id_product` AND sav.`id_product_attribute` = 0
        ' . StockAvailable::addSqlShopRestriction(null, null, 'sav') . ') ';

        $alias = 'sa';
        $alias_image = 'image_shop';
        $this->list_no_link = true;
        $id_shop = Shop::isFeatureActive() && Shop::getContext() == Shop::CONTEXT_SHOP ? (int) $this->context->shop->id : 'a.id_shop_default';
        $this->_join .= ' JOIN `' . _DB_PREFIX_ . 'product_shop` sa ON (a.`id_product` = sa.`id_product` AND sa.id_shop = ' . $id_shop . ')
                LEFT JOIN `' . _DB_PREFIX_ . 'wk_virtual_combination_available` wvca ON (a.`id_product` = wvca.`id_product`)
                LEFT JOIN `' . _DB_PREFIX_ . 'wk_virtual_combination_available_shop` wvcas
                ON (wvca.`id_wk_virtual_combination_available` = wvcas.`id_wk_virtual_combination_available` AND wvcas.`id_shop` = ' . (int) $this->context->shop->id . ')
                LEFT JOIN `' . _DB_PREFIX_ . 'category_lang` cl ON (' . $alias . '.`id_category_default` = cl.`id_category` AND b.`id_lang` = cl.`id_lang` AND cl.id_shop = ' . $id_shop . ')
                LEFT JOIN `' . _DB_PREFIX_ . 'shop` shop ON (shop.id_shop = ' . $id_shop . ')
                LEFT JOIN `' . _DB_PREFIX_ . 'image_shop` image_shop ON (image_shop.`id_product` = a.`id_product` AND image_shop.`cover` = 1 AND image_shop.id_shop = ' . $id_shop . ')
                LEFT JOIN `' . _DB_PREFIX_ . 'image` i ON (i.`id_image` = image_shop.`id_image`)
                LEFT JOIN `' . _DB_PREFIX_ . 'product_download` pd ON (pd.`id_product` = a.`id_product` AND pd.`active` = 1)';
        $this->_select .= 'wvcas.`availabe`, ';
        $this->_select .= 'shop.`name` AS `shopname`, a.`id_shop_default`, ';
        $this->_select .= $alias_image . '.`id_image` AS `id_image`, cl.`name` AS `name_category`, ' . $alias . '.`price`, 0 AS `price_final`, a.`is_virtual`, pd.`nb_downloadable`, sav.`quantity` AS `sav_quantity`, ' . $alias . '.`active`, IF(sav.`quantity`<=0, 1, 0) AS `badge_danger`';

        $this->_use_found_rows = false;
        $this->_group .= 'GROUP BY a.`id_product`';
        $this->_where .= ' AND a.`is_virtual` = 1';
        $this->fields_list = [];
        $this->fields_list['id_product'] = [
            'title' => $this->l('ID'),
            'align' => 'center',
            'class' => 'fixed-width-xs',
            'type' => 'int',
        ];
        $this->fields_list['image'] = [
            'title' => $this->l('Image'),
            'align' => 'center',
            'image' => 'p',
            'orderby' => false,
            'filter' => false,
            'search' => false,
        ];
        $this->fields_list['name'] = [
            'title' => $this->l('Name'),
            'filter_key' => 'b!name',
        ];
        $this->fields_list['reference'] = [
            'title' => $this->l('Reference'),
            'align' => 'left',
        ];
        $this->fields_list['availabe'] = [
            'title' => $this->l('Enable combination'),
            'align' => 'text-center',
            'type' => 'bool',
            'filter_key' => 'wvcas!availabe',
            'callback' => 'combinationStatusIcon',
            'search' => false,
            'filter' => false,
            'orderby' => false,
        ];
        if ($this->display == 'edit') {
            echo 'sd';
            exit;
        }
        if (Tools::getValue('id_product')) {
            $product = new Product((int) Tools::getValue('id_product'), false, $this->context->language->id);
            $this->page_header_toolbar_title = $product->name;
        }
    }

    public function initToolbar()
    {
        parent::initToolbar();
        unset($this->toolbar_btn['new']);
    }

    public function initProcess()
    {
        parent::initProcess();
        if (Tools::isSubmit('changeCombinationStatus') && $this->id_object) {
            if ($this->access('edit')) {
                $this->action = 'change_combination_status';
            } else {
                $this->errors[] = $this->l('You do not have permission to change this.');
            }
        }
    }

    public function processChangeCombinationStatus()
    {
        $idProduct = Tools::getValue('id_product');
        $idAvailable = WkVirtualCombinationAvailable::getAvailableId($idProduct, $this->context->shop->id);
        if (empty($idAvailable)) {
            $objProductAvailable = new WkVirtualCombinationAvailable();
        } else {
            $objProductAvailable = new WkVirtualCombinationAvailable($idAvailable);
        }
        $objProductAvailable->availabe = $objProductAvailable->availabe ? 0 : 1;
        $objProductAvailable->id_product = (int) $idProduct;
        $status = $objProductAvailable->save();

        if ($objProductAvailable->availabe) {
            WkVirtualCombinationAvailable::updateProductType($idProduct, ['product_type' => ProductType::TYPE_COMBINATIONS]);
        } else {
            WkVirtualCombinationAvailable::updateProductType($idProduct, ['product_type' => ProductType::TYPE_VIRTUAL]);
            // delete all combinations
            WkVirtualCombinationAvailable::deleteAllCombinations($idProduct);
        }
        if (!$status) {
            $this->errors[] = $this->l('An error occurred while updating.');
        } else {
            if ($objProductAvailable->availabe) {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminWkVirtualCombination') . '&conf=33&id_p=' . $idProduct);
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminWkVirtualCombination') . '&conf=34');
            }
        }
    }

    public function combinationStatusIcon($value, $object)
    {
        $data = [
            'value' => $value,
            'url' => $this->context->link->getAdminLink('AdminWkVirtualCombination') . '&id_product=' .
            (int) $object['id_product'] . '&changeCombinationStatus',
        ];
        $this->context->smarty->assign([
            'data' => $data,
            'wkcontroller' => Tools::getValue('controller'),
        ]);

        return $this->context->smarty->fetch(
            _PS_MODULE_DIR_ . $this->module->name .
            '/views/templates/admin/_partials/combination_status.tpl'
        );
    }

    private function boProductPageLink($idProduct = false)
    {
        return $this->context->link->getAdminLink(
            'AdminProducts',
            true,
            $idProduct ? ['id_product' => (int) $idProduct] : []
        );
    }

    /**
     * Function used to render the list to display for this controller.
     *
     * @return string|false
     *
     * @throws PrestaShopException
     */
    public function renderList()
    {
        $this->addRowAction('edit');
        if (!WkProductDownload::isEnableNewProductPage()) {
            $this->context->smarty->assign([
                'p_link' => $this->boProductPageLink(),
            ]);

            return $this->context->smarty->fetch(
                _PS_MODULE_DIR_ . $this->module->name . '/views/templates/admin/_partials/new_page_warning.tpl'
            );
        }

        return parent::renderList();
    }

    public function displayEditLink($token = null, $id)
    {
        if (WkVirtualCombinationAvailable::availableForVirtualCombi($id, $this->context->shop->id)) {
            $this->context->smarty->assign([
                'id_product' => $id,
            ]);

            return $this->context->smarty->fetch(
                _PS_MODULE_DIR_ . 'wkvirtualcombination/views/templates/admin/_partials/addvirtualfilelink.tpl'
            );
        }
    }

    public function renderForm()
    {
        if (!$this->loadObject(true)) {
            return;
        }
        $idProduct = Tools::getValue('id_product');
        if (!WkVirtualCombinationAvailable::availableForVirtualCombi($idProduct, $this->context->language->id)) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminWkVirtualCombination'));
        }
        $combinations = WkVirtualCombinationAvailable::getAllCominationsName($idProduct, $this->context->language->id);
        $this->context->smarty->assign([
            'bo_product_link' => $this->boProductPageLink($idProduct),
            'combinations' => $combinations,
            'id_product' => $idProduct,
            'max_upload_size' => Configuration::get('PS_LIMIT_UPLOAD_FILE_VALUE'),
            'virtual_upload_link' => $this->context->link->getAdminLink(
                'AdminWkVirtual',
                true
            ),
        ]);

        $this->fields_form = [
            'submit' => [
                'title' => $this->l('Save'),
                'class' => 'button',
            ],
        ];
        $form = parent::renderForm();

        return $form;
    }

    /**
     * Set assets for this controller
     *
     * @param bool $isNewTheme
     *
     * @return void
     */
    public function setMedia($isNewTheme = false)
    {
        parent::setMedia($isNewTheme);

        Media::addJsDef([
            'wk_edit' => $this->l('Edit'),
        ]);
        $this->addJS(_MODULE_DIR_ . $this->module->name . '/views/js/wkvirtualcombinationadmin_new.js');
    }
}
