<?php
/**
 * NOTICE OF LICENSE
 *
 * All right is reserved,
 * Please go through LICENSE.txt file inside our module
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future. If you wish to customize this module for your
 * needs please refer to CustomizationPolicy.txt file inside our module for more information.
 *
 * @author Webkul IN
 * @copyright since 2010 Webkul
 * @license LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminWkVirtualController extends ModuleAdminController
{
    public function __construct()
    {
        $this->bootstrap = true;
        $this->lang = false;
        $this->table = 'wk_product_download';
        $this->className = 'WkProductDownload';
        $this->identifier = 'id_wk_product_download';

        parent::__construct();
    }

    /**
     * Ajax server code
     * Function name with prefix ajaxProcess 'ajaxProcess[actionName]']
     *
     * @return json_encode or any other
     */
    public function ajaxProcessUpdateFile()
    {
        $productId = Tools::getValue('id_product');
        $attributeId = Tools::getValue('id_product_attribute');
        if (!empty($productId) && $attributeId) {
            $dbFileData = WkProductDownload::getWkProductDownload($productId, $attributeId);
            if (!empty($dbFileData['filename'])) {
                @unlink(_PS_DOWNLOAD_DIR_ . $dbFileData['filename']);
                WkProductDownload::updateWkProductDownload(
                    $productId,
                    $attributeId
                );
            }
        }
        $response['success']['id_product_attribute'] = $attributeId;
        $response['success']['msg'] = $this->l('Deleted.', 'adminwkvirtual');
        exit(json_encode($response));
    }

    public function ajaxProcessDeleteFile()
    {
        $productId = Tools::getValue('id_product');
        $attributeId = Tools::getValue('id_product_attribute');
        if (!empty($productId) && $attributeId) {
            $dbFileData = WkProductDownload::getWkProductDownload($productId, $attributeId);
            if (!empty($dbFileData) && isset($dbFileData['id_wk_product_download'])) {
                $download = new WkProductDownload($dbFileData['id_wk_product_download']);
                $download->delete();
            }
        }
        $response['success']['id_product_attribute'] = $attributeId;
        $response['success']['msg'] = $this->l('Updated.', 'adminwkvirtual');
        exit(json_encode($response));
    }

    /**
     * Ajax server code
     * Function name with prefix ajaxProcess 'ajaxProcess[actionName]']
     *
     * @return json_encode or any other
     */
    public function ajaxProcessPostData()
    {
        $response = [
            'errors' => [],
            'success' => [],
        ];
        $fileName = null;
        $virtualFileData = Tools::getValue('product_virtual_com');

        if (!empty($virtualFileData)) {
            $dbFileData = WkProductDownload::getWkProductDownload(
                $virtualFileData['id_product'],
                $virtualFileData['id_product_attribute']
            );
            $idWkProductDownload = null;

            if (!empty($dbFileData)) {
                $idWkProductDownload = $dbFileData['id_wk_product_download'];
                $fileName = $dbFileData['filename'];
            }

            $wkProductDownload = new WkProductDownload($idWkProductDownload ? $idWkProductDownload : null);

            if ((int) $virtualFileData['is_virtual_file'] == 1) {
                $fileCheck = $this->validateFile($fileName, $_FILES);

                if (array_key_exists('errors', $fileCheck) && !empty($fileCheck['errors'])) {
                    exit(json_encode($fileCheck));
                }
                if (isset($virtualFileData['name']) && empty($virtualFileData['name'])) {
                    $response['errors'] = [
                        'name' => $this->l('A file name is required', 'AdminWkVirtualController'),
                    ];
                    exit(json_encode($response));
                }
                if (Tools::strlen($virtualFileData['name']) > 255) {
                    $response['errors'] = [
                        'name' => $this->l('A file name is too long', 'AdminWkVirtualController'),
                    ];
                    exit(json_encode($response));
                }
                if (array_key_exists('filename', $fileCheck)) {
                    $fileName = $fileCheck['filename'];
                }

                $fieldErrors = $this->validateFields($virtualFileData);
                if (array_key_exists('errors', $fieldErrors) && !empty($fieldErrors['errors'])) {
                    exit(json_encode($fieldErrors));
                }

                $wkProductDownload->id_product = $virtualFileData['id_product'];
                $wkProductDownload->id_product_attribute = $virtualFileData['id_product_attribute'];
                $wkProductDownload->nb_days_accessible = (int) $virtualFileData['nb_days'];
                $wkProductDownload->nb_downloadable = (int) $virtualFileData['nb_downloadable'];
                $wkProductDownload->display_filename = trim($virtualFileData['name']);
                $wkProductDownload->filename = $fileName ? $fileName : $dbFileData['filename'];
                $wkProductDownload->date_expiration = $virtualFileData['expiration_date'] ?
                $virtualFileData['expiration_date'] . ' 23:59:59' : '';
                $wkProductDownload->is_virtual_file = (int) $virtualFileData['is_virtual_file'];
                $wkProductDownload->active = 1;

                if ($wkProductDownload->save()) {
                    $downloadLink = $this->context->link->getModuleLink('wkvirtualcombination', 'getfile');
                    $downloadLink = $downloadLink . '?&file=' . $fileName;
                    $response['success']['file_download_link'] = $downloadLink;
                    $response['success']['id_product_attribute'] = $virtualFileData['id_product_attribute'];
                    $response['success']['msg'] = $this->l('File updated.', 'AdminWkVirtualController');
                    exit(json_encode($response));
                }
            } else {
                if (!empty($idWkProductDownload)) {
                    $wkProductDownload->date_expiration = date('Y-m-d H:i:s', time() - 1);
                    $wkProductDownload->active = 0;
                    $wkProductDownload->update();
                }
            }
        }
    }

    protected function validateFields($virtualFileData)
    {
        $response = [
            'errors' => [],
        ];
        if (!Validate::isGenericName(trim($virtualFileData['name']))) {
            $response['errors'] = [
                'name' => $this->l('Filename is not valid.', 'AdminWkVirtualController'),
            ];

            return $response;
        }
        if (!Validate::isUnsignedInt(trim($virtualFileData['nb_downloadable']))) {
            $response['errors'] = [
                'nb_downloadable' => $this->l('Allowed downloads is not vaid.'),
            ];

            return $response;
        }
        if (!empty(trim($virtualFileData['expiration_date']))
            && !Validate::isDate(trim($virtualFileData['expiration_date']))) {
            $response['errors'] = [
                'expiration_date' => $this->l('Expiration date is not valid.', 'AdminWkVirtualController'),
            ];

            return $response;
        }
        if (!Validate::isUnsignedInt(trim($virtualFileData['nb_days']))) {
            $response['errors'] = [
                'nb_days' => $this->l('Number of days are not valid.', 'AdminWkVirtualController'),
            ];

            return $response;
        }

        return $response;
    }

    protected function validateFile($fileName, $file)
    {
        $response = [
            'errors' => [],
        ];
        if (empty($fileName)) {
            if (empty($file) && empty($fileName)) {
                $msg = $this->l('Please upload file.', 'AdminWkVirtualController');
                $response['errors'] = [
                    'file' => $msg,
                ];

                return $response;
            }

            if ((int) $file['product_virtual_com']['error']['file'] === 1) {
                $maxUpload = (int) ini_get('upload_max_filesize');
                $maxPost = (int) ini_get('post_max_size');
                $uploadMb = min($maxUpload, $maxPost);
                $message = str_replace(
                    '%upload_mb%',
                    $uploadMb,
                    $this->l(
                        'The file exceeds the size allowed by the server. The limit is set to %upload_mb% MB.',
                        'AdminWkVirtualController'
                    )
                );

                $response['errors'] = ['file' => $message];

                return $response;
            }

            if (isset($file['product_virtual_com']['tmp_name']['file'])
            && is_uploaded_file($file['product_virtual_com']['tmp_name']['file'])) {
                $psUploadLimit = Configuration::get('PS_LIMIT_UPLOAD_FILE_VALUE');
                if ($file['product_virtual_com']['size']['file'] > ($psUploadLimit * 1024 * 1024)) {
                    $psUploadLimit *= 1024;
                    $response['errors'] = [
                        'file' => str_replace(
                            '%ps_upload_limit%',
                            $psUploadLimit,
                            $this->l(
                                'The file is too large. Maximum size allowed is %ps_upload_limit% KB.',
                                'AdminWkVirtualController'
                            )
                        ),
                    ];

                    return $response;
                }
                if (Tools::strlen($file['product_virtual_com']['name']['file']) > 255) {
                    $response['errors'] = ['file' => $this->l('The file name is too long.', 'AdminWkVirtualController')];

                    return $response;
                }
                // $path_parts = pathinfo($file['product_virtual_com']['name']['file']);
                // $extension = Tools::strtolower($path_parts['extension']);
                // if (!in_array($extension, WkProductDownload::getFileTypes())) {
                //     $response['errors'] = array('file' => $this->module->l('File type not allowed.'));
                //     return $response;
                // }

                $fileName = ProductDownload::getNewFilename();
                $tmpFile = $file['product_virtual_com']['tmp_name']['file'];
                if (!move_uploaded_file($tmpFile, _PS_DOWNLOAD_DIR_ . $fileName)) {
                    $response['errors'] = ['file' => $this->l('Failed to upload the file.')];

                    return $response;
                }
                $response['filename'] = $fileName;
            }
        }

        return $response;
    }
}
