<?php
/**
 * NOTICE OF LICENSE
 *
 * All right is reserved,
 * Please go through LICENSE.txt file inside our module
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future. If you wish to customize this module for your
 * needs please refer to CustomizationPolicy.txt file inside our module for more information.
 *
 * @author Webkul IN
 * @copyright since 2010 Webkul
 * @license LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class WkVirtualCombinationGetFileModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();

        if (Tools::getValue('file')) {
            $filename = Tools::getValue('file');
            if (!Validate::isSha1($filename)) {
                exit(Tools::displayError());
            }
            $file = _PS_DOWNLOAD_DIR_ . (string) preg_replace('/\.{2,}/', '.', $filename);
            $filename = WkProductDownload::getFilenameFromFilename(Tools::getValue('file'));
            if (empty($filename)) {
                $newFileName = Tools::getValue('filename');
                if (!empty($newFileName)) {
                    $filename = Tools::getValue('filename');
                } else {
                    $filename = 'file';
                }
            }
            if (!file_exists($file)) {
                Tools::redirect('index.php');
            }

            if (ob_get_level() && ob_get_length() > 0) {
                ob_end_clean();
            }

            /* Set headers for download */
            header('Content-Transfer-Encoding: binary');
            header('Content-Type: application/octet-stream');
            header('Content-Length: ' . filesize($file));
            header('Content-Disposition: attachment; filename="' . $filename . '"');
            // prevents max execution timeout, when reading large files
            @set_time_limit(0);
            $fp = fopen($file, 'rb');

            if ($fp && is_resource($fp)) {
                while (!feof($fp)) {
                    echo fgets($fp, 16384);
                }
            }

            exit;
        }

        $errors = $this->validateFile();
        if ($errors != 1) {
            $this->context->smarty->assign([
                'message' => $errors,
                'redirect_url' => __PS_BASE_URI__,
            ]);
            $this->setTemplate('module:wkvirtualcombination/views/templates/front/alert.tpl');

            return true;
        }
    }

    protected function validateFile()
    {
        if (!($key = Tools::getValue('key'))) {
            $error = $this->displayCustomError('invalid_key');

            return $error;
        }

        Tools::setCookieLanguage();
        if (!$this->context->customer->isLogged()
            && !Tools::getValue('secure_key')
            && !Tools::getValue('id_order')) {
            Tools::redirect('index.php?controller=authentication&back=get-file.php&key=' . $key);
        } elseif (!$this->context->customer->isLogged()
            && Tools::getValue('secure_key')
            && Tools::getValue('id_order')) {
            $order = new Order((int) Tools::getValue('id_order'));
            if (!Validate::isLoadedObject($order)) {
                $error = $this->displayCustomError('invalid_key');

                return $error;
            }
            if ($order->secure_key != Tools::getValue('secure_key')) {
                $error = $this->displayCustomError('invalid_key');

                return $error;
            }
        }

        /* Key format: <sha1-filename>-<hashOrder> */
        $tmp = explode('-', $key);
        if (count($tmp) != 2) {
            $error = $this->displayCustomError('invalid_key');

            return $error;
        }

        $filename = $tmp[0];
        $hash = $tmp[1];
        if (!($info = WkVirtualOrder::getDownloadFromHash($hash))) {
            $error = $this->displayCustomError('this_product_does_not_exist_in_our_store');

            return $error;
        }

        /* check whether order has been paid, which is required to download the product */
        $orderDetail = new OrderDetail((int) $info['id_order_detail']);
        $order = new Order((int) $orderDetail->id_order);
        $state = $order->getCurrentOrderState();
        if (!$state || !$state->paid) {
            $error = $this->displayCustomError('this_order_has_not_been_paid');

            return $error;
        }

        if (!isset($info['id_product'])
            || empty($info['id_product'])
            || !isset($info['id_product_attribute'])
            || empty($info['id_product_attribute'])) {
            $error = $this->displayCustomError('this_product_has_been_deleted');

            return $error;
        }

        $idProductDownload = WkProductDownload::getWkProductDownload(
            $info['id_product'],
            $info['id_product_attribute']
        );
        if (!$idProductDownload) {
            $error = $this->displayCustomError('this_product_has_been_deleted');

            return $error;
        }

        if (!Validate::isFileName($idProductDownload['filename'])
        || !file_exists(_PS_DOWNLOAD_DIR_ . $idProductDownload['filename'])) {
            $error = $this->displayCustomError('this_file_no_longer_exists');

            return $error;
        }

        if (isset($orderDetail->product_quantity_refunded, $orderDetail->product_quantity_return)
            && ($orderDetail->product_quantity_refunded > 0
            || $orderDetail->product_quantity_return > 0)
        ) {
            $error = $this->displayCustomError('this_product_has_been_refunded');

            return $error;
        }

        $now = time();
        $productDeadline = strtotime($info['download_deadline']);
        if ($now > $productDeadline && $info['download_deadline'] != '0000-00-00 00:00:00') {
            $error = $this->displayCustomError('the_product_deadline_is_in_the_past');

            return $error;
        }

        $customerDeadline = strtotime($idProductDownload['date_expiration']);
        if ($now > $customerDeadline && $idProductDownload['date_expiration'] != '0000-00-00 00:00:00') {
            $error = $this->displayCustomError('expiration_date_has_passed_you_cannot_download_this_product');

            return $error;
        }

        if ($info['download_nb'] >= $idProductDownload['nb_downloadable'] && $idProductDownload['nb_downloadable']) {
            $error = $this->displayCustomError('you_have_reached_the_maximum_number_of_allowed_downloads');

            return $error;
        }

        if (Tools::getValue('ajax') == true) {
            return true;
        }
        /* Access is authorized -> increment download value for the customer */
        WkVirtualOrder::incrementDownload($info['id_wk_virtual_order']);
        $file = _PS_DOWNLOAD_DIR_ . $idProductDownload['filename'];
        $filename = $idProductDownload['display_filename'];

        if (ob_get_level() && ob_get_length() > 0) {
            ob_end_clean();
        }

        /* Set headers for download */
        header('Content-Transfer-Encoding: binary');
        header('Content-Type: application/octet-stream');
        header('Content-Length: ' . filesize($file));
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        // prevents max execution timeout, when reading large files
        @set_time_limit(0);
        $fp = fopen($file, 'rb');

        if ($fp && is_resource($fp)) {
            while (!feof($fp)) {
                echo fgets($fp, 16384);
            }
        }

        exit;
    }

    /**
     * Display an error message with js
     * and redirect using js function.
     *
     * @param string $msg
     */
    protected function displayCustomError($msg)
    {
        $translations = [
            'invalid_key' => $this->module->l('Invalid key.', 'getfile'),
            'this_product_does_not_exist_in_our_store' => $this->module->l('This product does not exist in our store.', 'getfile'),
            'this_product_has_been_deleted' => $this->module->l('This product has been deleted.', 'getfile'),
            'this_file_no_longer_exists' => $this->module->l('This file no longer exists.', 'getfile'),
            'this_product_has_been_refunded' => $this->module->l('This product has been refunded.', 'getfile'),
            'the_product_deadline_is_in_the_past' => $this->module->l('The product deadline is in the past.', 'getfile'),
            'expiration_date_has_passed_you_cannot_download_this_product' => $this->module->l('Expiration date has passed, you cannot download this product.', 'getfile'),
            'you_have_reached_the_maximum_number_of_allowed_downloads' => $this->module->l('You have reached the maximum number of downloads allowed.', 'getfile'),
        ];

        return $translations[$msg];
    }

    public function displayAjaxGetProductUrl()
    {
        $errors = $this->validateFile();
        if ($errors != 1) {
            echo json_encode([
                'title' => $this->module->l('Error: '),
                'message' => $errors,
                'status' => 0,
            ]);
        } else {
            echo json_encode([
                'redirect_url' => __PS_BASE_URI__,
                'status' => 1,
            ]);
        }
    }
}
