<?php
/**
 * NOTICE OF LICENSE
 *
 * All right is reserved,
 * Please go through LICENSE.txt file inside our module
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future. If you wish to customize this module for your
 * needs please refer to CustomizationPolicy.txt file inside our module for more information.
 *
 * @author Webkul IN
 * @copyright since 2010 Webkul
 * @license LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once 'classes/WkProductDownload.php';
require_once 'classes/WkVirtualCombinationDb.php';
require_once 'classes/WkVirtualOrder.php';
require_once 'classes/WkVirtualCombinationAvailable.php';

use PrestaShop\PrestaShop\Core\Domain\Product\ValueObject\ProductType;

class WkVirtualCombination extends Module
{
    public function __construct()
    {
        $this->name = 'wkvirtualcombination';
        $this->tab = 'front_office_features';
        $this->version = '4.1.0';
        $this->author = 'Webkul';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('Virtual product with combination');
        $this->description = $this->l('Admin can create virtual product with combination.
        ');
    }

    public function install()
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        $virtualComDb = new WkVirtualCombinationDb();
        if (!parent::install()
            || !$virtualComDb->createTable()
            || !$this->callInstallTab()
            || !$this->registerHook('actionAdminControllerSetMedia')
            || !$this->registerHook('displayAdminProductsCombinationBottom')
            || !$this->registerHook('displayOrderDetail')
            || !$this->registerHook('displayAdminProductsExtra')
            || !$this->registerHook('actionFrontControllerSetMedia')
            || !$this->registerHook('actionAfterUpdateProductFormHandler')
            || !$this->registerHook('actionOrderStatusPostUpdate')) {
            return false;
        } else {
            return true;
        }
    }

    public function hookDisplayAdminProductsExtra($params)
    {
        if (WkProductDownload::isEnableNewProductPage()) {
            $proObj = new Product($params['id_product']);
            if ($proObj->is_virtual) {
                $enableVirtualCombi = 0;
                if (WkVirtualCombinationAvailable::availableForVirtualCombi($params['id_product'], $this->context->shop->id)) {
                    $enableVirtualCombi = 1;
                }
                $this->context->smarty->assign(
                    [
                        'enable_virtual_combi' => $enableVirtualCombi,
                    ]
                );

                return $this->context->smarty->fetch(
                    _PS_MODULE_DIR_ . 'wkvirtualcombination/views/templates/hook/displayAdminProductsExtra.tpl'
                );
            }
        }
    }

    // update is_virtual to because on update PS change it when combination is enabled
    public function hookActionAfterUpdateProductFormHandler($params)
    {
        if (WkProductDownload::isEnableNewProductPage()) {
            $idProduct = $params['id'];
            if (Tools::getIsset('enableVirtualCombi')) {
                $enableVirtualCombi = Tools::getValue('enableVirtualCombi');
                $idAvailable = WkVirtualCombinationAvailable::getAvailableId($idProduct, $this->context->shop->id);
                if (empty($idAvailable)) {
                    $objProductAvailable = new WkVirtualCombinationAvailable();
                } else {
                    $objProductAvailable = new WkVirtualCombinationAvailable($idAvailable);
                }
                $objProductAvailable->availabe = (int) $enableVirtualCombi;
                $objProductAvailable->id_product = (int) $idProduct;
                $objProductAvailable->save();
                if ($enableVirtualCombi) {
                    WkVirtualCombinationAvailable::updateProductType($idProduct, ['product_type' => ProductType::TYPE_COMBINATIONS]);
                } else {
                    WkVirtualCombinationAvailable::updateProductType($idProduct, ['product_type' => ProductType::TYPE_VIRTUAL]);
                }
            }
            // this code to change is_virtual after save product
            if (WkVirtualCombinationAvailable::getAvailableId($idProduct, $this->context->shop->id)) {
                WkVirtualCombinationAvailable::updateProductType($idProduct, ['is_virtual' => 1]);
            }
        }
    }

    public function hookActionFrontControllerSetMedia()
    {
        if ($this->context->controller->php_self == 'order-detail'
        ) {
            $this->context->controller->registerJavascript(
                'wk_virtual_download_front',
                'modules/' . $this->name . '/views/js/wkvirtualfront.js'
            );
            $this->context->controller->addJqueryPlugin('growl', null, false);
            $this->context->controller->registerStylesheet('growl-css', 'js/jquery/plugins/growl/jquery.growl.css');
        }
    }

    public function callInstallTab()
    {
        // Parent hidden class
        $this->installTab('AdminWkVirtual', 'AdminWkVirtual', false, true);
        if (version_compare(_PS_VERSION_, '1.7.8.9', '>')) {
            $this->installTab('AdminWkVirtualCombination', 'Virtual Combination File', 'AdminCatalog');
        }

        return true;
    }

    public function installTab($className, $tabName, $tabParentName = false, $hidden = false)
    {
        $tab = new Tab();
        $tab->active = $hidden ? 0 : 1;
        $tab->class_name = $className;
        $tab->name = [];
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $tabName;
        }

        if ($tabParentName) {
            $tab->id_parent = (int) Tab::getIdFromClassName($tabParentName);
        } else {
            $tab->id_parent = 0;
        }

        $tab->module = $this->name;

        return $tab->add();
    }

    public function hookActionOrderStatusPostUpdate($status)
    {
        $order = new Order($status['id_order']);
        $orderPaid = $order->getCurrentOrderState() && $order->getCurrentOrderState()->paid;

        if ($orderPaid) {
            $orderProducts = $order->getCartProducts();
            foreach ($orderProducts as $orderProduct) {
                if ($orderPaid && $orderProduct['is_virtual']) {
                    $wkProductDownload = WkProductDownload::getWkProductDownload(
                        $orderProduct['product_id'],
                        $orderProduct['product_attribute_id']
                    );
                    if ($wkProductDownload && $wkProductDownload['active'] == 1) {
                        $virtualOrder = WkVirtualOrder::getVirtualOrder($orderProduct['id_order_detail']);
                        $virtualOrder->id_order_detail = $orderProduct['id_order_detail'];
                        $virtualOrder->id_product = $orderProduct['product_id'];
                        $virtualOrder->id_product_attribute = $orderProduct['product_attribute_id'];
                        $virtualOrder->download_hash = sha1(microtime());
                        $virtualOrder->download_nb = 0;
                        $virtualOrder->download_deadline = WkVirtualOrder::getDeadline(
                            $orderProduct['product_id'],
                            $orderProduct['product_attribute_id']
                        );
                        $virtualOrder->save();
                    }
                }
            }
        }
    }

    public function hookdisplayOrderDetail($params)
    {
        $order = $params['order'];
        if (isset($order)) {
            $orderPaid = $order->getCurrentOrderState() && $order->getCurrentOrderState()->paid;
            $orderProducts = $order->getCartProducts();
            $virtualOrder = [];
            $isVirtual = false;
            foreach ($orderProducts as $key => $orderProduct) {
                if ($orderPaid && $orderProduct['is_virtual']) {
                    $idProductDownload = WkProductDownload::getWkProductDownload(
                        $orderProduct['product_id'],
                        $orderProduct['product_attribute_id']
                    );
                    if (!empty($idProductDownload)
                        && $idProductDownload['active'] == 1
                        && $idProductDownload['is_virtual_file']
                    ) {
                        $product = new Product($orderProduct['product_id'], false, $this->context->language->id);
                        $attributeImage = Image::getBestImageAttribute(
                            $this->context->shop->id,
                            $this->context->language->id,
                            $orderProduct['product_id'],
                            $orderProduct['product_attribute_id']
                        );
                        if (Image::hasImages($this->context->language->id, $orderProduct['product_id'])) {
                            if (empty($attributeImage)) {
                                $image = Image::getCover($orderProduct['product_id']);
                                $imagePath = $this->context->link->getImageLink(
                                    $product->link_rewrite,
                                    $image['id_image'],
                                    ImageType::getFormattedName('small')
                                );
                            } else {
                                $imagePath = $this->context->link->getImageLink(
                                    $product->link_rewrite,
                                    $attributeImage['id_image'],
                                    ImageType::getFormattedName('small')
                                );
                            }
                        } else {
                            $imagePath = '';
                        }

                        $virtualOrder[$key]['name'] = $orderProduct['product_name'];
                        $virtualOrder[$key]['image'] = $imagePath;
                        if ($idProductDownload) {
                            $wkProductDownload = new WkProductDownload($idProductDownload['id_wk_product_download']);
                            $downloadLink = $wkProductDownload->getTextLink($orderProduct['id_order_detail']);
                            if ($downloadLink) {
                                $virtualOrder[$key]['download_link'] = $downloadLink . '&id_order=' . (int) $order->id
                                . '&secure_key=' . $order->secure_key;

                                $virtualOrder[$key]['front_download_link'] = explode('&key=', $downloadLink)[1];
                                $virtualOrder[$key]['front_id_order'] = (int) $order->id;
                                $virtualOrder[$key]['front_secure_key'] = $order->secure_key;
                                $virtualOrder[$key]['GetProductUrl'] = $this->context->link->getModuleLink('wkvirtualcombination', 'getfile');
                            }
                        }
                        $isVirtual = true;
                    }
                }
            }
            if ($isVirtual) {
                $this->context->smarty->assign(['order' => $virtualOrder]);

                return $this->display(__FILE__, 'order-detail.tpl');
            }
        }
    }

    protected function getFileUploadSize()
    {
        $maxUpload = (int) ini_get('upload_max_filesize');
        $maxPost = (int) ini_get('post_max_size');
        $uploadMb = min($maxUpload, $maxPost);

        return $uploadMb;
    }

    public function hookActionAdminControllerSetMedia()
    {
        $controllerName = Tools::getValue('controller');
        if ($controllerName == 'AdminProducts') {
            $uploadMb = $this->getFileUploadSize();
            Media::addJsDef([
                'module_js_dir' => _MODULE_DIR_ . $this->name . '/views/js/',
                'inactive_virtual_file_message' => $this->l('Do you want to inactive file?'),
                'wkConfirmation' => $this->l('Are you sure to delete this?'),
                'wkVirtualProductJS' => $this->l('Virtual product'),
                'upload_file_error' => str_replace(
                    '%upload_mb%',
                    $uploadMb,
                    $this->l(
                        'The file exceeds the size allowed by the server. The limit is set to %upload_mb% MB.'
                    )
                ),
            ]);
            $this->context->controller->addJS(
                $this->_path . 'views/js/wkvirtualcombination.js'
            );
        }
    }

    public function hookDisplayAdminProductsCombinationBottom($params)
    {
        $product = new Product($params['id_product'], false, $this->context->language->id);

        if ($product && $product->is_virtual == 1) {
            $virtualFile = WkProductDownload::getWkProductDownload(
                $params['id_product'],
                $params['id_product_attribute']
            );
            $downloadLink = '';
            if (!empty($virtualFile) && !empty($virtualFile['filename'])) {
                $downloadLink = $this->context->link->getModuleLink('wkvirtualcombination', 'getfile')
                . '?&file=' .
                $virtualFile['filename'];
            }
            $this->context->smarty->assign([
                'max_upload_size' => Configuration::get('PS_LIMIT_UPLOAD_FILE_VALUE'),
                'virtual_upload_link' => $this->context->link->getAdminLink(
                    'AdminWkVirtual',
                    true
                ),
                'id_product' => $params['id_product'],
                'id_product_attribute' => $params['id_product_attribute'],
                'virtual_file_data' => $virtualFile,
                'download_link' => $downloadLink,
            ]);

            return $this->display(__FILE__, 'virtual.tpl');
        }
    }

    public function uninstall()
    {
        $virtualComDb = new WkVirtualCombinationDb();
        if (!parent::uninstall()
            || !$this->uninstallTab()
            || !$virtualComDb->deleteTables()) {
            return false;
        }

        return true;
    }

    /**
     * Uninstall admin tabs
     *
     * @return bool
     */
    public function uninstallTab()
    {
        $moduleTabs = Tab::getCollectionFromModule($this->name);
        if (!empty($moduleTabs)) {
            foreach ($moduleTabs as $moduleTab) {
                $moduleTab->delete();
            }
        }

        return true;
    }

    // Get combination virtual product for email download link
    public function getVirtualProductEmail($orderProducts, $order)
    {
        $virtualOrder = [];
        foreach ($orderProducts as $key => $orderProduct) {
            if ($orderProduct['is_virtual']) {
                $idProductDownload = WkProductDownload::getWkProductDownload(
                    $orderProduct['product_id'],
                    $orderProduct['product_attribute_id']
                );
                if (!empty($idProductDownload)
                    && $idProductDownload['active'] == 1
                    && $idProductDownload['is_virtual_file']
                ) {
                    $product = new Product($orderProduct['product_id'], false, $this->context->language->id);
                    $attributeImage = Image::getBestImageAttribute(
                        $this->context->shop->id,
                        $this->context->language->id,
                        $orderProduct['product_id'],
                        $orderProduct['product_attribute_id']
                    );
                    if (empty($attributeImage)) {
                        $image = Image::getCover($orderProduct['product_id']);
                        $imagePath = $this->context->link->getImageLink(
                            $product->link_rewrite,
                            $image['id_image'],
                            ImageType::getFormattedName('small')
                        );
                    } else {
                        $imagePath = $this->context->link->getImageLink(
                            $product->link_rewrite,
                            $attributeImage['id_image'],
                            ImageType::getFormattedName('small')
                        );
                    }

                    $virtualOrder[$key]['name'] = $orderProduct['product_name'];
                    $virtualOrder[$key]['image'] = $imagePath;

                    if ($idProductDownload) {
                        $wkProductDownload = new WkProductDownload($idProductDownload['id_wk_product_download']);
                        $downloadLink = $wkProductDownload->getTextLink($orderProduct['id_order_detail']);
                        if ($downloadLink) {
                            $virtualOrder[$key]['download_link'] = $downloadLink . '&id_order=' . (int) $order->id
                            . '&secure_key=' . $order->secure_key;
                        }
                    }
                }
            }
        }

        return $virtualOrder;
    }
}
